#! /usr/bin/env python
import json
import os
import sys

from scrib import barf


curdir = os.getcwd()
confdir = os.path.expanduser('~') + '/.scrib/bots/'


def get_name(argv):
	name = 'scrib'  # set default name
	if len(argv) is 2:
		name = argv[1]

	dir_check(confdir)

	with open(confdir + 'current', 'w+') as f:
		f.write(name)
	return name


def dir_check(thisdir):
	if not os.path.exists(thisdir):
		os.makedirs(thisdir)


class set:
	def __init__(self):
		
		self.name = get_name(sys.argv)
		self.scrib_dir = confdir + self.name + '/'
		dir_check(self.scrib_dir)
		
	def load(self, filename, defaults, suppress=False):
		"""
		Load scrib settings.
		"""
		self._defaults = defaults
		self._filename = filename

		# try to load saved ones
		vars = self.__load_config(filename, suppress)

		if vars is None:
			# none found. this is new
			vars = self._defaults
			self.save()
			self.load(filename, self._defaults, True)
			return
		for i in vars.keys():
			self.__dict__[i] = vars[i]
		self._defaults = vars

	def save(self, suppress=False):
		"""
		Save scrib settings
		"""
		self.__save_config(self._filename, self._defaults, True)

	def __load_config(self, filename, suppress=False):
		try:
			f = open(self.scrib_dir + filename, 'r')
		except IOError:
			return None

		try:
			config = json.load(f)
			f.close()
		except:
			barf.Barf('ERR', '%s%s could not be loaded.' % (self.scrib_dir, filename))
			barf.Barf('ERR', 'Please make sure it is properly formatted.')
			sys.exit()

		if suppress is False:
			barf.Barf('MSG', self.scrib_dir + filename + ' successfully loaded.')

		return config

	def __save_config(self, filename, fields, suppress=False):
		"""
		This saves a dictionary to json
		"""
		f = ''
		with open(self.scrib_dir + filename, 'w') as out:
			f = f + str(json.dump(fields, out, indent=4))

		if suppress is False:
			barf.Barf('SAV', self.scrib_dir + filename + ' successfully saved.')
