#! /usr/bin/env python
import sys

if sys.version_info[0] >= 3:
	from scrib import barf, cfg, clean, db
else:
	import barf
	import cfg
	import clean
	import db


class Process:
	"""
	Setting up processing.
	https://github.com/coleifer/irc/blob/master/bots/redisbot.py
	http://code.activestate.com/recipes/194364-the-markov-chain-algorithm/
	"""
	def __init__(self, interface):
		self.settings = cfg.set()
		# Don't set defaults here.
		self.settings.load('scrib.cfg', {}, suppress=True)
		self.interface = cfg.set()
		# Don't set defaults here.
		self.interface.load(interface + '.cfg', {}, suppress=True)
		self.barf = barf.Barf
		self.db = db.Db()
		self.clean = clean.Clean(self.settings.debug)
		self.messages_to_generate = 1  # Default is 5
		self.separator = ' '
	
	def msg(self, data, learn, reply=''):
		messages = []
		if self.settings.debug is 1:
			self.barf('DBG', 'process.msg: %s' % data)

		sentences = self.split_to_sentences(data)
		
		for sentence in sentences:
			sentence = self.clean.split(self.clean.sanitize(sentence))
			for words in sentence:
				if self.settings.debug is 1:
					self.barf('DBG', 'process.msg: for loop fired')
				key = self.separator.join(words[:-1])
				if learn is True:
					self.db.add(key, words[-1])

				best_message = ''
				for i in range(self.messages_to_generate):
					generated = self.generate(seed=key)
					if len(generated) > len(best_message):
						best_message = generated

				if best_message:
					messages.append(best_message)

		if len(messages) and reply:
			if self.settings.debug is 1:
				self.barf('DBG', 'process.msg: replying')
			return max(messages, key=len)

		if self.settings.debug is 1:
				self.barf('DBG', 'process.msg: not replying')

	def split_to_sentences(self, message):
		message = str(message)
		try:
			try:
				message = [x for x in map(str.strip, message.split('.')) if x]
			except AttributeError as e:
				self.barf('ERR', 'process.split_to_sentences: AttributeError: %s' % e)
				
			return message
		except Exception as e:
			self.barf('ERR', 'process.split_to_sentences: %s' % e)

	def info(self):
		return self.db().info()

	def generate(self, seed):
		"""
		GENERATE ALL THE THINGS
		"""
		if self.settings.debug is 1:
			self.barf('DBG', 'process.generate: Seed: %s ' % seed)
		key = seed
		gen_words = []

		for i in xrange(self.interface.maxwords):
			words = key.split(self.separator)
			gen_words.append(words[0])
			next_word = self.db.get_random(key)
			if self.settings.debug is 1:
				self.barf('DBG', 'process.generate: Next word: %s' % next_word)
			if not next_word:
				break

			key = self.separator.join(words[1:] + [next_word])

		return ' '.join(gen_words)
